local VERSION = tonumber(("$Revision: 42376 $"):match("%d+"))

CowTip = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceEvent-2.0", "AceConsole-2.0", "FuBarPlugin-2.0", "AceHook-2.1", "AceModuleCore-2.0")
local CowTip, self = CowTip, CowTip
CowTip.version = "r" .. VERSION
CowTip.revision = VERSION
CowTip.date = ("$Date: 2007-07-01 02:38:34 -0400 (Sun, 01 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")

local waterfall = AceLibrary:HasInstance("Waterfall-1.0") and AceLibrary("Waterfall-1.0")

local _G = _G
local GameTooltip = _G.GameTooltip
local geterrorhandler = _G.geterrorhandler
local pcall = _G.pcall
local ipairs = _G.ipairs
local InCombatLockdown = _G.InCombatLockdown
local UIParent = _G.UIParent
local rawget = _G.rawget
local pairs = _G.pairs
local setmetatable = _G.setmetatable
local type = _G.type

local localeTables = {}
function CowTip:L(name)
	if not localeTables[name] then
		localeTables[name] = setmetatable({}, {__index = function(self, key)
			self[key] = key
			return key
		end})
	end
	return localeTables[name]
end

local L = CowTip:L("CowTip")

CowTip:RegisterDB("CowTipDB")
CowTip:RegisterDefaults('profile', {
	unitShow = "always",
	unitFrameShow = "always",
	otherFrameShow = "always",
})

-- FuBar configuration
CowTip.hasIcon = true
CowTip.hideWithoutStandby = true
CowTip.defaultPosition = "MINIMAP"
CowTip.defaultMinimapPosition = 230
CowTip.hasNoText = true
CowTip.hasNoColor = true
CowTip.cannotDetachTooltip = true
CowTip.independentProfile = true
CowTip.blizzardTooltip = true

local modulesWithFunction = setmetatable({}, {__index = function(self, key)
	local t = {}
	for name, module in CowTip:IterateModules() do
		if type(module[key]) == "function" and CowTip:IsModuleActive(module) then
			t[#t+1] = module
		end
	end
	self[key] = t
	
	return t
end})

function CowTip:OnModuleCreated(name, module)
	for k in pairs(modulesWithFunction) do
		modulesWithFunction[k] = nil
	end
end

local leftGameTooltipStrings = {}
local rightGameTooltipStrings = {}
CowTip.leftGameTooltipStrings = leftGameTooltipStrings
CowTip.rightGameTooltipStrings = rightGameTooltipStrings

function CowTip:OnInitialize()
	self:RegisterChatCommand('/CowTip', '/Cow', {
		type = 'execute',
		func = function()
			self:OpenMenu()
		end
	})
	
	GameTooltip:SetOwner(UIParent, "ANCHOR_NONE")
	for i = 1, 50 do
		GameTooltip:AddDoubleLine(' ', ' ')
		leftGameTooltipStrings[i] = _G["GameTooltipTextLeft" .. i]
		rightGameTooltipStrings[i] = _G["GameTooltipTextRight" .. i]
	end
	GameTooltip:Show()
	GameTooltip:Hide()
	
	if self.db.profile.texts and self:HasModule("Text") then
		self:GetModule("Text").db.lefts = self.db.profile.texts.lefts
		self:GetModule("Text").db.rights = self.db.profile.texts.rights
		self.db.profile.texts = nil
	end
	
	for k,v in pairs(self.db.profile) do
		for name, module in self:IterateModules() do
			if module.db then
				local db = module.db.profile
				if rawget(db, k) then
					db[k] = v
					self.db.profile[k] = nil
					break
				end
			end
		end
	end
end

function CowTip:OnEnable()
	self:HookScript(GameTooltip, "OnShow", "GameTooltip_OnShow")
	self:HookScript(GameTooltip, "OnHide", "GameTooltip_OnHide")
	self:HookScript(GameTooltip, "OnTooltipSetUnit", "GameTooltip_OnTooltipSetUnit")
	
	self:Hook(GameTooltip, "FadeOut", "GameTooltip_FadeOut", true)
end

function CowTip:OnDisable()
	self:GameTooltip_OnHide(GameTooltip)
end

function CowTip:OnModuleEnable()
	for k in pairs(modulesWithFunction) do
		modulesWithFunction[k] = nil
	end
end

function CowTip:OnModuleDisable()
	for k in pairs(modulesWithFunction) do
		modulesWithFunction[k] = nil
	end
end

function CowTip:OnClick()
	if waterfall then
		waterfall:Open("CowTip")
	else
		self:Print(L["Waterfall-1.0 is required to access the GUI."])
	end
end

local function OnTooltipSetUnit(self)
	local unit
	if not GameTooltip:IsOwned(UIParent) then
		local _
		_, unit = GameTooltip:GetUnit()
	else
		unit = "mouseover"
	end

	for _,v in ipairs(modulesWithFunction.PreTooltipSetUnit) do
		local success, ret = pcall(v.PreTooltipSetUnit, v)
		if not success then
			geterrorhandler()(ret)
		end
	end

	for _,v in ipairs(modulesWithFunction.OnTooltipSetUnit) do
		local success, ret = pcall(v.OnTooltipSetUnit, v)
		if not success then
			geterrorhandler()(ret)
		end
	end
end

local doneOnTooltipMethod = false

local forgetNextOnTooltipSetUnit = false
function CowTip:GameTooltip_OnTooltipSetUnit(this, ...)
	doneOnTooltipMethod = true
	self.hooks[this].OnTooltipSetUnit(this, ...)
	
	if forgetNextOnTooltipSetUnit then
		forgetNextOnTooltipSetUnit = false
	else
		OnTooltipSetUnit(self)
	end
end

function CowTip:GameTooltip_OnShow(this, ...)
	if not doneOnTooltipMethod then
		if this:GetUnit() then
			OnTooltipSetUnit(self)
			forgetNextOnTooltipSetUnit = true
		end
	end
	local show
	if this:IsOwned(UIParent) then
		-- world unit
		show = self.db.profile.unitShow
	elseif this:GetUnit() then
		-- unit frame
		show = self.db.profile.unitFrameShow
	else
		-- non-unit frame
		show = self.db.profile.otherFrameShow
	end
	
	if show == "noncombat" then
		if InCombatLockdown() then
			this:Hide()
			return
		end
	elseif show == "never" then
		this:Hide()
		return
	end
	self.hooks[this].OnShow(this, ...)
	
	for _,v in ipairs(modulesWithFunction.OnTooltipShow) do
		local success, ret = pcall(v.OnTooltipShow, v)
		if not success then
			geterrorhandler()(ret)
		end
	end
end

function CowTip:GameTooltip_FadeOut(this, ...)
	self.hooks[this].FadeOut(this, ...)
end

function CowTip:GameTooltip_OnHide(this, ...)
	doneOnTooltipMethod = false
	
	for _,v in ipairs(modulesWithFunction.OnTooltipHide) do
		local success, ret = pcall(v.OnTooltipHide, v)
		if not success then
			geterrorhandler()(ret)
		end
	end
	
	self.hooks[this].OnHide(this, ...)
end

local showWhenValidate = {
	never = L["Never"],
	notcombat = L["Out of combat"],
	always = L["Always"],
}

CowTip.options = {
	type = 'group',
	handler = CowTip,
	args = {
		unitShow = {
			name = L["Show unit tooltip"],
			desc = L["Show the tooltip for world units if..."],
			type = 'text',
			validate = showWhenValidate,
			get = function()
				return CowTip.db.profile.unitShow
			end,
			set = function(value)
				CowTip.db.profile.unitShow = value
			end,
		},
		unitFrameShow = {
			name = L["Show unit frame tooltip"],
			desc = L["Show the tooltip for unit frames if..."],
			type = 'text',
			validate = showWhenValidate,
			get = function()
				return CowTip.db.profile.unitFrameShow
			end,
			set = function(value)
				CowTip.db.profile.unitFrameShow = value
			end,
		},
		otherFrameShow = {
			name = L["Show non-unit frame tooltip"],
			desc = L["Show the tooltip for non-unit frames if..."],
			type = 'text',
			validate = showWhenValidate,
			get = function()
				return CowTip.db.profile.otherFrameShow
			end,
			set = function(value)
				CowTip.db.profile.otherFrameShow = value
			end,
		},
	}
}

function CowTip.modulePrototype:RegisterCowTipOption(data)
	local args = CowTip.options.args
	local name = self.name
	if args[name] then
		local i = 1
		while args[name .. i] do
			i = i + 1
		end
		name = name .. i
	end
	args[name] = data
	if not data.handler then
		data.handler = self
	end
	if data.args then
		data.args.active = {
			type = 'toggle',
			name = L["Enable"],
			desc = L["Enable this module"],
			get = "IsModuleActive",
			set = "ToggleModuleActive",
			handler = CowTip,
			passValue = self,
			order = 1,
		}
	end
end

if waterfall then
	waterfall:Register("CowTip", 
						"aceOptions", CowTip.options,
						"title", L["CowTip Configuration"],
						"treeLevels", 5);
end

CowTip.OnMenuRequest = CowTip.options

function CowTip:OnTooltipUpdate()
	GameTooltip:AddLine("CowTip")
	GameTooltip:AddLine(" ")
	--[[
	GameTooltip:AddDoubleLine(LOCKED .. ":", (self.db.profile.settings.locked and YES or NO), 1, 1, 0, 1, 1, 1)
	local text
	if configMode == "solo" then
		text = L["Solo"]
	elseif configMode == "party" then
		text = L["Party"]
	elseif configMode == "raid" then
		text = L["Raid"]
	else
		text = L["Off"]
	end
	GameTooltip:AddDoubleLine(L["Configuration mode"] .. ":", text, 1, 1, 0, 1, 1, 1)
	GameTooltip:AddLine(" ")
	GameTooltip:AddLine(L["|cffffff00Click|r to lock/unlock unit frames."], 0.2, 1, 0.2)
	GameTooltip:AddLine(L["|cffffff00Shift-Click|r to change configuration mode."], 0.2, 1, 0.2)
	]]
	if waterfall then
		GameTooltip:AddLine(L["|cffffff00Left-Click|r to change settings with a nice GUI configuration."], 0.2, 1, 0.2)
	end
	GameTooltip:AddLine(L["|cffffff00Right-Click|r to change settings with a drop-down menu."], 0.2, 1, 0.2)
end
